Program Morse;
{
   02/26/90

   Morse Code Program. Submitted to public domain by
   Jerry Adkins KB8YA/6    COMPUSERVE ID: 70455,112.
   7770 Regents Rd. #226
   San Diego, CA 92122

   This program makes use of direct array indexing, instead of using a
   lookup table. This makes very high translation speeds possible.
   You may experiment with the typed constants HZ and PAUSE for different
   default tone and speed.

   The files CQ.TXT and QSO.TXT reflect a typical conversation you are
   likely to hear on the novice bands.

   You may optionally play a text file from DOS by typing:
   morse filename.ext

   It was done as a one evening experiment. Enjoy.
}

Uses
   Crt;

Type
   String6 = String[6];

CONST
   ESC   = #27;
   HZ    : Integer = 800;   { HZ and PAUSE are typed constants that are }
   PAUSE : Integer = 200;   { subject to change. }
   DONE  = FALSE;
   SETUPNAME = 'MORSE.SET'; { File that holds changed HZ and PAUSE }

 { The array MORSECHAR has indices corresponding to the ASCII
     value of the chars normally used in the Morse alphabet. This
     allows high speed translation without using lookup tables. }
   MORSECHAR : Array[44..90] Of String6 =
     ('--..--' { , },    '-...-'  { - },
      '.-.-.-' { . },    '-..-.'  { / },
      '-----'  { 0 },    '.----'  { 1 },
      '..---'  { 2 },    '...--'  { 3 },
      '....-'  { 4 },    '.....'  { 5 },
      '-....'  { 6 },    '--...'  { 7 },
      '---..'  { 8 },    '----.'  { 9 },
      '','','','','',
      '..--..' { ? }     ,'',
      '.-'     { A },    '-...'   { B },
      '-.-.'   { C },    '-..'    { D },
      '.'      { E },    '..-.'   { F },
      '--.'    { G },    '....'   { H },
      '..'     { I },    '.---'   { J },
      '-.-'    { K },    '.-..'   { L },
      '--'     { M },    '-.'     { N },
      '---'    { O },    '.--.'   { P },
      '--.-'   { Q },    '.-.'    { R },
      '...'    { S },    '-'      { T },
      '..-'    { U },    '...-'   { V },
      '.--'    { W },     '-..-'  { X },
      '-.--'   { Y },     '--..'  { Z });

   LEGALCHARS : Set Of Char = [#44..#57,#63,#65..#90];

Var
   Cmd : Integer;

Procedure SoundOut(n:Byte);
{ ***********************************************************
  Store MORSECHAR[n] in Element string. Scan through string
  and sound speaker based on a dot or dash. Use set Frequency
  in HZ and delay time in PAUSE.
  ***********************************************************}
Var
   i : Byte;
   Element : String6;

Begin
   Element := MORSECHAR[n];
   For i := 1 to Length(Element) Do
   Begin
      Case Element[i] Of
         '-' : Begin
                  Sound(HZ);
                  Delay(PAUSE);
               End;
         '.' : Begin
                  Sound(HZ);
                  Delay(PAUSE Div 2);
               End;
      End; { Case }
      NoSound;
      Delay(PAUSE Div 3);
   End; { i }
   Delay(PAUSE);
End; { SoundOut }

Procedure OutPutChar(Ch:Char);
{ ***********************************************************
  Convert Ch to number and send result to Procedure SoundOut
  ***********************************************************}
Var
   n : Byte;

Begin
   If Ch <> ESC Then
   Write(Ch);
   If Ch = #32 Then
   Delay(PAUSE); { Delay for space }
   n := Ord(Ch);
   If Not (Chr(n) In LEGALCHARS) Then Exit; { Not morse char }
   SoundOut(n);
End; { OutPutChar }

Procedure MorseKey;
{ ***********************************************************
  Input from keyboard and process keystrokes. Exit on ESC.
  ***********************************************************}
Var
   Ch : Char;

Begin
   ClrScr;
   WriteLn('Start typing. Press ESC to exit.');
   Repeat
      Ch := ReadKey;
      If Ch = #13 Then Writeln;
      Ch := UpCase(Ch);
      OutPutChar(Ch);
   Until Ch = ESC;
End; { MorseKey }

Procedure OutPutLine(Line:String; Var Abort:Boolean);
{ ***********************************************************
  Read text line from a file and process each char for output.
  ***********************************************************}
Var
   i : Byte;
   Ch : Char;

Begin
   For i := 1 to Length(Line) Do
   Begin
      If KeyPressed Then
      Begin
         Ch := ReadKey;
         If Ch = ESC Then
         Begin
            Abort := True;
            Exit;
         End;
      End;
      Ch := UpCase(Line[i]);
      OutPutChar(Ch);
   End;
   WriteLn;
End; { OutPutLine }

Procedure MorseRead(FileName:String);
{ ***********************************************************
  Open text file and send to speaker as morse code.
  ***********************************************************}
Var
   TextFile : Text;
   Line : String;
   Err : Integer;
   Ch  : Char;
   Abort : Boolean;

Begin
   Abort := False;
   If FileName = '' Then
   Begin
      ClrScr;
      GoToXY(20,12);
      Write('Filename: ');
      ReadLn(FileName);
   End; { If }
   If FileName = '' Then Exit;
   {$I-}
   WriteLn;
   Assign(TextFile,FileName);
   Reset(TextFile);
   Err := IoResult;
   {$I+}
   If Err <> 0 Then
   Begin
      GoToXY(20,13);
      Write(#7,FileName,' not found. Press ENTER. ');
      ReadLn;
      Exit;
   End;
   ClrScr;
   WriteLn('Reading ',FileName,'. Press ESC to abort.');
   While Not Eof(TextFile) Do
   Begin
      ReadLn(TextFile,Line);
      OutputLine(Line,Abort);
      If Abort Then
      Begin
         Close(TextFile);
         Exit;
      End;
   End;
   Close(TextFile);
End; { MorseRead }

Procedure RandomCode;
{ ***********************************************************
  Random code practice in sets of four chars.
  ***********************************************************}
Var
   n, c : Byte;
   Ch : Char;

Begin
   n := 0;
   Ch := #0;
   Randomize;
   ClrScr;
   WriteLn('Random code practice. Press ESC to abort.');
   Repeat
      If KeyPressed Then Ch := ReadKey;
      Repeat
        c := Random(90);
      Until Chr(c) In LEGALCHARS; { Acceptable Morse Chars }
      Write(Chr(c));
      SoundOut(c);
      Inc(n);
      If n > 3 Then
      Begin
         Write(#32);
         Delay(PAUSE);
         n := 0;
      End;
   Until Ch = ESC;
End; { RandomCode }

Procedure CodeTest;
{ ***********************************************************
  Random code sent. User types char in response to sound.
  ***********************************************************}
Var
   Ch : Char;
   n  : Byte;
   Ok : Boolean;
   CharsTyped, CharsMissed, Score : Real;

Begin
   CharsTyped  := 0;
   CharsMissed := 0;
   Randomize;
   ClrScr;
   WriteLn('Type your response after hearing the sound. Press ESC to exit.');
   Repeat
      Repeat
         n := Random(90);
      Until (n > 43) And (Chr(n) In LEGALCHARS);
      SoundOut(n);
      Ch := UpCase(ReadKey);
      If Ch > Esc Then
      Begin
         CharsTyped := CharsTyped + 1;
         If Ord(Ch) = n Then
         Begin
            TextColor(White);
            Write(Ch);
         End Else
         Begin
            TextColor(Red);
            Write('   ',Chr(n));
            CharsMissed := CharsMissed + 1;
         End;
         WriteLn;
      End;
   Until Ch = ESC;
   Score := (CharsMissed/CharsTyped) * 100;
   TextColor(White);
   ClrScr;
   GoToXY(1,12);
   If (CharsTyped > 1) And (CharsMissed < 1) Then
    Write('Congratulations!!! Your score is 100%') Else
   Begin
      Write('You missed ',CharsMissed:3:0,' Out of ');
      Write(CharsTyped:4:0);
      Write('. Your score is ',100-Score:3:1,'%');
   End;
   Write(' . Press ENTER. ');
   ReadLn;
End; { CodeTest }

Procedure Settings;
{ ***********************************************************
  Get Frequency in HZ.
  Get PAUSE time. PAUSE time is 1000th of a second.
  ***********************************************************}
Var
   TextFile : Text;

Begin
   ClrScr;
   Repeat
      GoToXY(20,12); Write('Frequency in HZ ',HZ,': ');
      ReadLn(HZ);
   Until HZ > 200;
   Repeat
      GoToXY(20,14); Write('Delay time between chars ',PAUSE,': ');
      ReadLn(PAUSE);
   Until PAUSE > 10;
   Assign(TextFile,SetupName);
   ReWrite(TextFile);
   WriteLn(TextFile,HZ);
   WriteLn(TextFile,PAUSE);
   Close(TextFile);
End; { Settings }

Procedure LoadSettings(Var HZ,PAUSE:Integer);
{ ***********************************************************
  Load previous HZ and PAUSE settings if they exist.
  ***********************************************************}
Var
   TextFile : Text;
   Err : Integer;

Begin
   {$I-}
   Assign(TextFile,SetupName);
   Reset(TextFile);
   Err := IoResult;
   {$I+}
   If Err <> 0 Then Exit;
   ReadLn(TextFile,HZ);
   ReadLn(TextFile,PAUSE);
   Close(TextFile);
End; { LoadSettings }

Function Menu:Integer;
{ ***********************************************************
  Display menu and return with number selection.
  ***********************************************************}
Var
   Choice : Integer;
   Ch : Char;

Begin
   Repeat
      TextColor(White);
      TextBackground(Black);
      ClrScr;
      GoToXY(20,7);
      Write('MORSE CODE PROGRAM. RELEASED TO PUBLIC DOMAIN BY');
      GoToXY(20,8);
      Write('JERRY ADKINS KB8YA/6  COMPUSERVE ID: 70455,112');

      GoToXY(30,10); Write('1 - CODE PRACTICE');
      GoToXY(30,12); Write('2 - READ TEXT FILE');
      GoToXY(30,14); Write('3 - RANDOM CODE');
      GoToXY(30,16); Write('4 - SET SPEED AND TONE');
      GoToXY(30,18); Write('5 - CODE TEST');
      GoToXY(30,20); Write('Press ESC to exit.');
      Ch := ReadKey;
      If Ch = #0 Then
      Ch := ReadKey;
      Case Ch Of
         ESC :
         Begin
            Menu := 0;
            Exit;
         End;
        '1'..'5' :
        Begin
           Menu := Ord(Ch)-48;
           Exit;
        End;
     End; { Case }
  Until DONE;
End; { Menu }

Begin { Main Procedure }
{ ***********************************************************
  Main part of program. Control branching and exit to DOS.
  ***********************************************************}
   LoadSettings(HZ,PAUSE);
   If ParamCount > 0 Then
   Begin
      MorseRead(ParamStr(1));
      Halt(0);
   End;
   Repeat
      Cmd := Menu;
      Case Cmd Of
         1: MorseKey;
         2: MorseRead('');
         3: RandomCode;
         4: Settings;
         5: CodeTest;
      End; { Case }
   Until Cmd = 0;
End. { Morse }
